async function checkHref(elem, token, authProvider) {
  const msg = document.getElementById("msg")
  const resourceName = elem.getAttribute("resource-name")
  const resourceId = elem.getAttribute("resource-id")
  const resourceProviderId = elem.getAttribute("resource-provider-id") ? elem.getAttribute("resource-provider-id") : ""
  const fqResourceName = resourceProviderId ? `${resourceProviderId}:${resourceName}` : resourceName
  const url = resourceId ? `resource/${fqResourceName}/${resourceId}` : `resource/${fqResourceName}`
  const resp = await fetch(url, {
    method: "GET",
    headers: new Headers({
      "Content-type": "application/json",
      "Authorization": `Bearer ${token}`,
      "auth_provider": authProvider,
    }),
  }).catch(err => {
    msg.innerHTML = "Cannot fetch resource: " + err.message
    resourceElem.innerHTML = ""
  })
  if (resp === undefined) {
    return
  } else {
    elem.classList.add("hasResponseStatus")
    elem.classList.add("status-" + resp.status)
    elem.title = "Response code: " + resp.status + " - " + resp.statusText
  }
}

function checkPerms(className, token, authProvider) {
  var rootElems = document.getElementsByClassName(className)
  Array.from(rootElems).forEach(elem =>
    Array.from(elem.children).forEach(elem => checkHref(elem, token, authProvider))
  )
}

async function get_resource(resourceName, token, authProvider, resourceId, resourceProviderId) {
  // BaseUrl for an external resource provider
  //if (!keycloak.keycloak) { return }
  const msg = document.getElementById("msg")
  const resourceElem = document.getElementById('resource')
  const fqResourceName = resourceProviderId ? `${resourceProviderId}:${resourceName}` : resourceName
  const url = resourceId ? `resource/${fqResourceName}/${resourceId}` : `resource/${fqResourceName}`
  const resp = await fetch(url, {
    method: "GET",
    headers: new Headers({
      "Content-type": "application/json",
      "Authorization": `Bearer ${token}`,
      "auth_provider": authProvider,
    }),
  }).catch(err => {
    msg.innerHTML = "Cannot fetch resource: " + err.message
    resourceElem.innerHTML = ""
  })
  if (resp === undefined) {
    return
  }
  const resource = await resp.json()
  if (!resp.ok) {
    msg.innerHTML = resource["detail"]
    resourceElem.innerHTML = ""
    return
  }
  msg.innerHTML = ""
  resourceElem.innerHTML = ""
  Object.entries(resource).forEach(
    ([key, value]) => {
      let r = document.createElement('div')
      let kElem = document.createElement('div')
      kElem.innerText = key
      kElem.className = "key"
      let vElem = document.createElement('div')
      if (typeof value == "object") {
        Object.entries(value).forEach(v => {
          const ne = document.createElement('div')
          ne.innerHTML = `<span class="key">${v[0]}</span>: <span class="value">${v[1]}</span>`
          vElem.appendChild(ne)
        })
      }
      else {
        vElem.innerText = value
      }
      vElem.className = "value"
      if (key == "sorry") {
        vElem.classList.add("error")
      }
      r.appendChild(kElem)
      r.appendChild(vElem)
      resourceElem.appendChild(r)
    }
  )
}
